function [qmod, lmod, inter] = rsquad(D)
% RSQUAD - Construct data matrices related to quadratic models. Version 0.1.22.
%
% The syntax is
% [qmod, lmod, inter] = rsquad(D)
%
% Arguments
% D is an N by k design matrix with factor data.
%
% Return values
% qmod is the data matrix for a full quadratic model corresponding to D.
% lmod is the corresponding data matrix for a linear 
% pred is the response predicted by the model at the stationary point
% 
%
% References
% Neter, Wasserman, Kutner: Applied Linear Regression Models, R.D. Irwin Inc., 1983, Chapter 9.


% Copyright (C) 2016 Mikael Kurula <mkurula@abo.fi>
%
% This program is free software; you can redistribute it and/or modify it under
% the terms of the GNU General Public License as published by the Free Software
% Foundation; either version 3 of the License, or (at your option) any later
% version.
%
% This program is distributed in the hope that it will be useful, but WITHOUT
% ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
% FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
% details.
%
% You should have received a copy of the GNU General Public License along with
% this program; if not, see <http://www.gnu.org/licenses/>.
%
% Author: Mikael Kurula <mkurula@abo.fi>, 2016


% Check input
if nargin ~= 1
    error('Try help rsquad for instructions.') ;
end

% Build data matrix for linear model
[N,k]=size(D);
lmod=[ones(N,1) D];

% Build interactions part
rot = @(A,n) [A(:,n+1:end) A(:,1:n)];
inter=[];

for j=1:(ceil(k/2)-1)
  inter=[inter D.*rot(D,j)];
end  
if mod(k,2)==0,
  DrotDtmp=D.*rot(D,k/2);
  inter=[inter DrotDtmp(:,1:(k/2))];
end

% Assemble data matrix of full quadratic model
qmod=[lmod D.^2 inter];

end